/**
 * Fixed Engine. 
 * 
 * Fixes geometry errors, and switches to a hierarchical rendering system for simplicity's sake
 * Still lacks a way to click properly-rotated/scaled children, but this shouldn't be an issue yet
 * Would be easier to fix with broader support of currentTransform, but can be worked around by duplicating matrix math if it proves necessary
 *
 * @author Andrew Fraticelli <andrewf@meetfidel.com>
 */
;(function(window){
	
	var degToRad = Math.PI/180;
	
	function renderChild(curChild,curX,curY,stillClickable,ctx,pre,post,includeComps){
		
		if (curChild.alpha <= 0) return;
		
		var isComplex = false;
		if (
			(curChild.scaleX != 1) ||
			(curChild.scaleY != 1) ||
			(curChild.rotation != 0) ||
			(curChild.skewX != 0) ||
			(curChild.skewY != 0)
			) {
			
			isComplex = true;
			ctx.save();
			
			stillClickable = false;
			// once the ancestry involves more than just x/y offsets, clickability cannot be calculated without lots of extra math.
			// this can be somewhat avoided once ctx.currentTransform is more widely supported, but it will still be costly...
			// but it's unlikely that complexly-transformed elements will be used for menu items anyway
			// (MAYBE scaling, and that would be the simplest one to implement anyway... might be worth it?)
		}
		
		var oldAlpha = null;
		if (curChild.alpha != 1) {
			oldAlpha = ctx.globalAlpha;
			ctx.globalAlpha *= curChild.alpha;
		}
		
		curX += curChild.x;
		curY += curChild.y;
		
		if (isComplex) {
			ctx.translate(curX,curY);
			curX = 0;
			curY = 0;
		}
		
		if (curChild.rotation != 0) ctx.rotate(curChild.rotation*degToRad);
		if (curChild.skewX) ctx.transform(1, 0, Math.tan(curChild.skewX), 1, 0, 0);
		if (curChild.skewY) ctx.transform(1, Math.tan(curChild.skewY), 0, 1, 0, 0);
		if ( (curChild.scaleX != 1) || (curChild.scaleY != 1) ) ctx.scale(curChild.scaleX, curChild.scaleY);
		
		// adapted from old render function:
		
		if (pre) pre(curChild);
		
		if(curChild.data && curChild.data != null){
			if(!curChild.inComposition || (includeComps && curChild.inComposition)){
				ctx.drawImage(curChild.data, curChild.xOffset, curChild.yOffset, curChild.dataWidth, curChild.dataHeight, Math.round(curX), Math.round(curY), curChild.width, curChild.height);
			}
		}
		
		if (post) post(curChild);
		
		//
		
		if (curChild.children) {// it seems like some displayobjects don't have children, is that normal?
			for(var i=0; i<curChild.children.length; i++) {
				renderChild(curChild.children[i],curX,curY,stillClickable,ctx,pre,post,includeComps);
			}
		}
		
		if (oldAlpha !== null) ctx.globalAlpha = oldAlpha;// faster than save/restore (though it may be redundant with one if there are other transforms; but an assign won't change much)
		
		if (isComplex) ctx.restore();
		
	}

	/**
	 * AMD Closure
	 */	
	if ( typeof define === "function" && define.amd ) {
		define( "sdkcustom/FixedEngine", ["sdk/Engine"], function (Engine) {
			
			//FixedEngine.Inherit(Engine);
			function FixedEngine(tag, _debug){
				Arstider.Super(this, Engine,tag, _debug);
				//this.Inherit(Engine, tag, _debug);
			}
			
			FixedEngine.prototype.render = function(list, ctx, pre, post, includeComps){
				this.captureTarget.update();
				renderChild(this.captureTarget,0,0,true,ctx,pre,post,includeComps);
			};
			
			return FixedEngine;
			
		});
	}
	else{
		rjsError("FixedEngine");
	}
})(window);